********************************************************************************************
***DISCERN/ ASHISH ARORA, SHARON BELENZON, LIA SHEER (DUKE UNIVERSITY) / JANUARY 2020***
********************************************************************************************
*Compiling Patent data: flow+stock variables including dynamic reassignment of patents 
*Match file of UO & Subsidiary Patents- linking id_name to Patents: "patent_match_id_name.dta"
******************************************************************************************************************
*******************************************************************************************************************
********************************************************************************************************************
**********************************************************************************************************
**********RELEVANT PERMNO_ADJ AT PATENT GRANT YEAR***************
*Dynamic list UO and Subsidiary
use "./data/DISCERN_UO_name_list.dta", clear
append using "./data/DISCERN_SUB_name_list.dta"
keep sample id_name fyear* nyear* permno_adj*
label var sample "U=UO+Labs+traded-Subs//A=ORBIS Subs//B=Pre-2002 manual Subs//C=NBER2006"
save "./data/dyn_match_All.dta", replace

use "./data/patent_match_id_name.dta"  , clear
merge m:m id_name sample using "./data/dyn_match_All.dta"
keep if _m==3
drop _m
ren publn_year year
* appropriate permno_adj to assign the patents in grant year
gen permno_adj=.
forvalue i=0/5 {
replace permno_adj = permno_adj`i' if permno_adj`i'~=. & year>=fyear`i' & year<=nyear`i'
}
drop if permno_adj==.
ren year  publn_year 
label var permno_adj "UO firm at the time patent was granted"
save "./data/patent_1980_2015.dta", replace
***********COMPUTING PATENT STOCK - similar to NBER2006 code***************
use "./data/patent_1980_2015.dta"  , clear
drop permno_adj
duplicates drop

sort publn_nr
* count number of assignees for each patent
by publn_nr: egen nass = count(id_name1)


* calculate fractional patent ownership
gen npat = 1 / nass


*gen NPAT, for each ID_NAME-YEAR

sort id_name1 publn_year
by id_name1 publn_year: egen npat_id_name_gyear=sum(npat)

save "./data/patent_database", replace

use "./data/patent_database", clear
duplicates drop id_name1  publn_year, force
keep id_name1 publn_year npat_id_name_gyear
ren publn_year year
*name level pat count
fillin id_name1 year
replace npat_id_name_gyear=0 if npat_id_name_gyear==.

*Generating patent stock - 0.15% growth per year plus 15% depreciation
gen sum_npat_id_name=npat_id_name_gyear/0.15
sort id_name1 year
by id_name1: replace sum_npat_id_name=0.85*sum_npat_id_name[_n-1]+npat_id_name_gyear if sum_npat_id_name[_n-1]!=.

gen id_name=substr(id_name1, 2,.)
gen sample=substr(id_name1, 1,1)
destring id_name, replace

*dynamic match of assignee to compustat companies
merge m:m id_name sample using "./data/dyn_match_All.dta"
keep if _m==3
drop _m

* appropriate permno_adj to assign the patents in each year

gen permno_adj=.
gen endyr=.


forvalue i=0/5 {
replace permno_adj = permno_adj`i' if permno_adj`i'~=. & year>=fyear`i' & year<=nyear`i'
replace endyr = nyear`i' if nyear`i'~=. & year>=fyear`i' & year<=nyear`i'
}

keep year id_name1 id_name npat_id_name_gyear _fillin sum_npat_id_name  permno_adj endyr sample
save "./data/id_name_year_npat", replace



* sum over multiple assignees to get patents for each permno_adj company
use "./data/id_name_year_npat", clear
drop if permno_adj==.
sort id_name1 year
collapse (sum) sum_npat_id_name,by(permno_adj year)
ren sum_npat_id_name pat_stock_permno_adj
label var pat_stock_permno_adj "pat stock by permno_adj"
save "./data/pat_stock_permno_adj", replace
*******************************************************************************************
*Patent firms- with positive patent stock

use "./data/pat_stock_permno_adj", clear
drop if pat_stock_permno_adj==0
keep permno_adj
duplicates drop
save "./data/patent_firms.dta" , replace

**************************COMPUTING PATENTS PER YEAR***********************************
use "./data/patent_database", clear
duplicates drop id_name1 publn_year, force
keep id_name1 id_name publn_year npat_id_name_gyear sample
ren publn_year year

merge m:m id_name sample using "./data/dyn_match_All.dta"
keep if _m==3
drop _m


* appropriate permno_adj to assign the patents in each year
gen permno_adj=.
forvalue i=0/5 {
replace permno_adj = permno_adj`i' if permno_adj`i'~=. & year>=fyear`i' & year<=nyear`i'
}
keep if permno_adj~=.
keep permno_adj year npat_id_name_gyear

* sum over multiple id_name to get patents for each company-year
sort permno_adj year
collapse (sum) npat_id_name_gyear,by(permno_adj year)

fillin permno_adj year
replace npat_id_name_gyear=0 if npat_id_name_gyear==.

ren npat_id_name_gyear pat_yr
drop _fillin
save "./data/pat_per_year_permno_adj", replace
************************************************************************************************************
